@@ -31,7 +31,7 @@ if __name__ == '__main__':
 #     parser.add_argument('--concat', action='store_true')
     parser.add_argument('--projection', '-p', default=None)
     parser.add_argument('--impute', action='store_true')
-    parser.add_argument('--outdir', '-o', type=str, default='result')
+    parser.add_argument('--outdir', '-o', type=str, default='output/')
     
     parser.add_argument('--lr', type=float, default=2e-4)
     parser.add_argument('--batch_size', type=int, default=64)
@@ -60,5 +60,5 @@ if __name__ == '__main__':
                    seed=args.seed, 
                    gpu=args.gpu, 
                    outdir=args.outdir, 
-                   project=args.project, 
-                   save=True)
\ No newline at end of file
+                   projection=args.projection, 
+                   save=True)
