@@ -13,6 +13,9 @@ import scipy
 from tqdm import tqdm
 
 from torch.utils.data import Dataset
+from torch.utils.data.sampler import Sampler
+from torch.utils.data import DataLoader
+
 from anndata import AnnData
 import scanpy as sc
 from sklearn.preprocessing import maxabs_scale, MaxAbsScaler
@@ -25,6 +28,9 @@ DATA_PATH = os.path.expanduser("~")+'/.scalex/'
 
 
 def read_mtx(path):
+    """
+    read mtx format data folder
+    """
     for filename in glob(path+'/*'):
         if ('count' in filename or 'matrix' in filename or 'data' in filename) and ('mtx' in filename):
             adata = sc.read_mtx(filename).T
@@ -32,14 +38,19 @@ def read_mtx(path):
         if 'barcode' in filename:
             barcode = pd.read_csv(filename, sep='\t', header=None).iloc[:, -1].values
             adata.obs = pd.DataFrame(index=barcode)
-        if 'gene' in filename or 'peaks' in filename or 'feature' in filename:
+        if 'gene' in filename or 'peaks' in filename:
             gene = pd.read_csv(filename, sep='\t', header=None).iloc[:, -1].values
             adata.var = pd.DataFrame(index=gene)
+        elif 'feature' in filename:
+            gene = pd.read_csv(filename, sep='\t', header=None).iloc[:, 1].values
+            adata.var = pd.DataFrame(index=gene)
     return adata
 
 
 def load_file(path):  
-    if os.path.isdir(path): # mtx format
+    if os.path.exists(DATA_PATH+path+'.h5ad'):
+        adata = sc.read_h5ad(DATA_PATH+path+'.h5ad')
+    elif os.path.isdir(path): # mtx format
         adata = read_mtx(path)
     elif os.path.isfile(path):
         if path.endswith(('.csv', '.csv.gz')):
@@ -59,6 +70,9 @@ def load_file(path):
 
 
 def load_files(root):
+    """
+    load single cell dataset given the data path
+    """
     if root.split('/')[-1] == '*':
         adata = []
         for root in sorted(glob(root)):
@@ -70,6 +84,11 @@ def load_files(root):
     
 def concat_data(data_list, batch_categories=None, join='inner', 
                 batch_key='batch', index_unique=None, save=None):
+    """
+    Concat multiple datasets
+    """
+    if len(data_list) == 1:
+        return load_files(data_list[0])
     adata_list = []
     for root in data_list:
         adata = load_files(root)
@@ -86,14 +105,6 @@ def concat_data(data_list, batch_categories=None, join='inner',
     return concat
         
     
-def load_dataset(name, path=DATA_PATH):
-    if os.path.exists(path+name+'.h5ad'):
-        adata = sc.read_h5ad(path+name+'.h5ad')
-        return adata
-    else:
-        raise ValueError('No such dataset named {} under {}'.format(name, path))
-    
-        
 class SingleCellDataset(Dataset):
     """
     Dataset for dataloader
@@ -110,26 +121,12 @@ class SingleCellDataset(Dataset):
         domain_id = self.adata.obs['batch'].cat.codes[idx]
         return x, domain_id, idx
     
-
-def standard_scale(X, vmax=5):
-    if type(X) != np.ndarray:
-        X = X.toarray()
-    X = scale(X)
-    X[X>vmax]=vmax
-    X = minmax_scale(X)
-    return scipy.sparse.csr_matrix(X)
-
-
-def filter_data(adata, min_genes=600, min_cells=3):
-    sc.pp.filter_cells(adata, min_genes=min_genes)
-    sc.pp.filter_genes(adata, min_cells=min_cells)
-
     
 def preprocessing(adata, 
         min_genes=600, 
         min_cells=3, 
         target_sum=1e4, 
-        n_top_genes=None, # or gene list
+        n_top_genes=2000, # or gene list
         transform=maxabs_scale,
         split=True,
     ):
@@ -140,31 +137,23 @@ def preprocessing(adata,
         adata.X = scipy.sparse.csr_matrix(adata.X)
     
     adata = adata[:, [gene for gene in adata.var_names 
-                  if not str(gene).startswith(tuple(['ERCC', 'MT-', 'mt-']))]].copy()
+                  if not str(gene).startswith(tuple(['ERCC', 'MT-', 'mt-']))]]
     
     sc.pp.filter_cells(adata, min_genes=min_genes)
     sc.pp.filter_genes(adata, min_cells=min_cells)
     sc.pp.normalize_total(adata, target_sum=target_sum, exclude_highly_expressed=True)
     sc.pp.log1p(adata)
     
-    if n_top_genes is not None:
-        if type(n_top_genes) == int:
-            if n_top_genes>0:
-                print('Find highly variable genes')
-                df = sc.pp.highly_variable_genes(adata, n_top_genes=n_top_genes, batch_key='batch', inplace=False) #, subset=True)
-                adata = transform_data(adata, transform, split=split)
-                raw = adata.copy()
-                adata._inplace_subset_var(df['highly_variable'].values)
-        else:
-            adata = transform_data(adata, transform, split=split)
-            raw = adata.copy()
-            if type(n_top_genes) == str:
-                n_top_genes = np.loadtxt(n_top_genes, dtype=str)
-            adata = reindex(adata, n_top_genes)
-
-    adata.raw = raw 
+    if type(n_top_genes) == int and n_top_genes>0:
+#         if n_top_genes>0:
+#             print('Find {} highly variable genes'.format(n_top_genes))
+        sc.pp.highly_variable_genes(adata, n_top_genes=n_top_genes, batch_key='batch', inplace=False, subset=True)
+    else:
+        adata = reindex(adata, n_top_genes)
+        
+    return transform_data(adata, transform, split=split)
             
-    return adata
+#     return adata
     
 
 def transform_data(adata, transform, split=True):
@@ -191,10 +180,68 @@ def reindex(adata, genes):
     return adata
    
     
-def down_sample(adata, cat='celltype', size=500):
-    indices = []
-    for c in adata.obs[cat].cat.categories:
-        index = adata[adata.obs[cat]==c].obs_names
-        idx = list(np.random.choice(index, size=min(size, len(index)), replace=False))
-        indices+=idx
-    return adata[indices]
\ No newline at end of file
+# def down_sample(adata, cat='celltype', size=500):
+#     indices = []
+#     for c in adata.obs[cat].cat.categories:
+#         index = adata[adata.obs[cat]==c].obs_names
+#         idx = list(np.random.choice(index, size=min(size, len(index)), replace=False))
+#         indices+=idx
+#     return adata[indices]
+
+
+class splitBatchSampler(Sampler):
+    """
+    split multi-datasets Batch Sampler
+    sampled data of each batch is from the same dataset.
+    """
+    def __init__(self, batch_size, batch_id, drop_last=False):
+        self.batch_size = batch_size
+        self.drop_last = drop_last
+        self.batch_id = batch_id
+
+    def __iter__(self):
+        batch = {}
+        sampler = np.random.permutation(len(self.batch_id))
+        for idx in sampler:
+            c = self.batch_id[idx]
+            if c not in batch:
+                batch[c] = []
+            batch[c].append(idx)
+
+            if len(batch[c]) == self.batch_size:
+                yield batch[c]
+                batch[c] = []
+
+        for c in batch.keys():
+            if len(batch[c]) > 0 and not self.drop_last:
+                yield batch[c]
+            
+    def __len__(self):
+        if self.drop_last:
+            return len(self.batch_id) // self.batch_size
+        else:
+            return (len(self.batch_id)+self.batch_size-1) // self.batch_size
+
+
+def load_dataset(data_list, batch_categories=None, join='inner', batch_key='batch', batch_name='batch',
+                 min_genes=600, min_cells=3, n_top_genes=2000, batch_size=64):
+    adata = concat_data(data_list, batch_categories, join=join, batch_key=batch_key)
+    print('Raw dataset shape: {}'.format(adata.shape))
+    if batch_name!='batch':
+        adata.obs['batch'] = adata.obs[batch_name]
+    if 'batch' not in adata.obs:
+        adata.obs['batch'] = 'batch'
+    adata.obs['batch'] = adata.obs['batch'].astype('category')
+    
+    adata = preprocessing(adata, 
+                          min_genes=min_genes, 
+                          min_cells=min_cells, 
+                          n_top_genes=n_top_genes,
+                         )
+    scdata = SingleCellDataset(adata)
+    trainloader = DataLoader(scdata, batch_size=batch_size, 
+                                 drop_last=True, shuffle=True, num_workers=4)
+    batch_sampler = splitBatchSampler(batch_size, adata.obs['batch'], drop_last=False)
+    testloader = DataLoader(scdata, batch_sampler=batch_sampler)
+    
+    return adata, trainloader, testloader 
\ No newline at end of file
