@@ -35,7 +35,7 @@ def SCALEX(data_list,
            max_iteration=30000,
            seed=124, 
            gpu=0, 
-           outdir='result/', 
+           outdir='output/', 
            projection=None, 
            impute=None, 
            save=True):
@@ -83,7 +83,8 @@ def SCALEX(data_list,
         model.load_model(projection+'/checkpoint/model.pt')
         model.to(device)
         
-        adata, trainloader, testloader = load_dataset(data_list, batch_categories, join='outer', n_top_genes=n_top_genes)
+        adata, trainloader, testloader = load_dataset(data_list, batch_categories, join='outer', 
+                                                      n_top_genes=n_top_genes, min_cells=0)
         log.info('Processed dataset shape: {}'.format(adata.shape))
         
     adata.obsm['latent'] = model.encodeBatch(testloader, device=device) # save latent rep
